@@ -3,30 +3,31 @@
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY.  No author or distributor
-;; accepts responsibility to anyone for the consequences of using it
-;; or for whether it serves any particular purpose or works at all,
-;; unless he says so in writing.  Refer to the GNU Emacs General Public
-;; License for full details.
+;; GNU Emacs is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 1, or (at your option)
+;; any later version.
 
-;; Everyone is granted permission to copy, modify and redistribute
-;; GNU Emacs, but only under the conditions described in the
-;; GNU Emacs General Public License.   A copy of this license is
-;; supposed to have been given to you along with GNU Emacs so you
-;; can know your rights and responsibilities.  It should be in a
-;; file named COPYING.  Among other things, the copyright notice
-;; and this notice must be preserved on all copies.
+;; GNU Emacs is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+
+;; You should have received a copy of the GNU General Public License
+;; along with GNU Emacs; see the file COPYING.  If not, write to
+;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 
 ;; The following line is modified automatically
 ;; by loading inc-version.el, each time a new Emacs is dumped.
-(defconst emacs-version "18.41.0" "\
+(defconst emacs-version "18.59.0" "\
 Version numbers of this version of Emacs.")
 
 (defconst emacs-build-time (current-time-string) "\
 Time at which Emacs was dumped out.")
 
+(defconst emacs-build-system (system-name))
+
 (defun emacs-version () "\
 Return string describing the version of Emacs that is running."
   (interactive)
@@ -37,7 +38,7 @@ Return string describing the version of Emacs that is running."
 	    (substring emacs-build-time 0
 		       (string-match " *[0-9]*:" emacs-build-time))
 	    (substring emacs-build-time (string-match "[0-9]*$" emacs-build-time))
-	    (system-name) system-type)))
+	    emacs-build-system system-type)))
 
 ;;Local variables:
 ;;version-control: never
