@@ -18,25 +18,55 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 
+/* This must precede sys/signal.h on certain machines.  */
+#include <sys/types.h>
 #include <signal.h>
 #include <errno.h>
 
 #include "config.h"
-#include <stdio.h>
+#ifdef NULL
 #undef NULL
+#endif
 #include "lisp.h"
+#undef NULL
 #include "commands.h"
 
-#include <sys/types.h>
+/* Get FIONREAD, if it is available,
+   just to help decide whether SIGIO should be defined.  */
+#ifdef USG
+#include <termio.h>
+#include <fcntl.h>
+#else /* not USG */
+#ifndef VMS
+#include <sys/ioctl.h>
+#endif /* not VMS */
+#endif /* not USG */
+
+/* Allow m- file to inhibit use of FIONREAD.  */
+#ifdef BROKEN_FIONREAD
+#undef FIONREAD
+#endif
+
+/* We are unable to use interrupts if FIONREAD is not available,
+   so flush SIGIO so we won't try. */
+#ifndef FIONREAD
+#ifdef SIGIO
+#undef SIGIO
+#endif
+#endif
+
+#include <stdio.h>
 #include <sys/file.h>
 
 #ifdef VMS
 #include <ssdef.h>
 #endif
 
+#if 0 /* fcntl.h was included above.  */
 #ifdef USG5
 #include <fcntl.h>
 #endif
+#endif
 
 #ifdef BSD
 #include <sys/ioctl.h>
@@ -173,11 +203,35 @@ init_cmdargs (argc, argv, skip_args)
 #ifdef VMS
 #ifdef LINK_CRTL_SHARE
 #ifdef SHAREABLE_LIB_BUG
+#ifdef __GNUC__
+#define	environ $$PsectAttributes_NOSHR$$environ
+extern char **environ;
+#else
 extern noshare char **environ;
+#endif
 #endif /* SHAREABLE_LIB_BUG */
 #endif /* LINK_CRTL_SHARE */
 #endif /* VMS */
 
+/* We don't include crtbegin.o and crtend.o in the link,
+   so these functions and variables might be missed.
+   Provide dummy definitions to avoid error.
+   (We don't have any real constructors or destructors.)  */
+#ifdef __GNUC__
+#ifndef ORDINARY_LINK
+__do_clobal_ctors ()
+{}
+__do_clobal_ctors_aux ()
+{}
+__do_global_dtors ()
+{}
+char * __CTOR_LIST__[2] = { (char *) (-1), 0 };
+char * __DTOR_LIST__[2] = { (char *) (-1), 0 };
+__main ()
+{}
+#endif /* not ORDINARY_LINK */
+#endif /* __GNUC__ */
+
 /* ARGSUSED */
 main (argc, argv, envp)
      int argc;
@@ -193,13 +247,13 @@ main (argc, argv, envp)
   if (argc > 1 && !strcmp (argv[1], "-nl"))
     {
       map_in_data (0);
-      /* The shared momory was just restored, which clobbered this.  */
+      /* The shared memory was just restored, which clobbered this.  */
       skip_args = 1;
     }
   else
     {
       map_in_data (1);
-      /* The shared momory was just restored, which clobbered this.  */
+      /* The shared memory was just restored, which clobbered this.  */
       skip_args = 0;
     }
 #endif
@@ -232,6 +286,15 @@ main (argc, argv, envp)
     brk (bss_end);
 #endif
 
+#ifdef NeXT
+  extern int malloc_cookie;
+
+  /* This helps out unexnext.c.  */
+  if (initialized)
+    if (malloc_jumpstart (malloc_cookie) != 0)
+      printf ("malloc jumpstart failed!\n");
+#endif /* NeXT */
+
   clearerr (stdin);
 
 #ifdef APOLLO
@@ -300,6 +363,16 @@ main (argc, argv, envp)
       noninteractive = 1;
     }
 
+#ifdef POSIX_SIGNALS
+  init_signals ();
+#endif
+
+#ifdef HAVE_TZSET
+  /* Reinitialize the time zone if it was initialized before dumping Emacs.  */
+  if (initialized)
+    tzset ();
+#endif
+
   if (
 #ifndef CANNOT_DUMP
       ! noninteractive || initialized
@@ -332,15 +405,25 @@ main (argc, argv, envp)
 #endif SIGXFSZ
 
 #ifdef AIX
-      signal (SIGDANGER, fatal_error_signal);
+      /* This used to run fatal_error_signal,
+	 but it isn't fatal.  There's nothing Emacs can usefully do.
+	 Might as well let the system kill us if it insists.  */
+      signal (SIGDANGER, SIG_IGN);
       signal (20, fatal_error_signal);
       signal (21, fatal_error_signal);
       signal (22, fatal_error_signal);
       signal (23, fatal_error_signal);
       signal (24, fatal_error_signal);
+#ifdef SIGIO
       signal (SIGAIO, fatal_error_signal);
       signal (SIGPTY, fatal_error_signal);
-      signal (SIGIOINT, fatal_error_signal);
+#endif
+#ifdef SIGURG
+      /* Note that SIGIOINT is the same as SIGIO on some machines,
+	 and the same as SIGURG on others.  It seems ore reliable to use the
+	 name with a uniform meaning.  */
+      signal (SIGURG, fatal_error_signal);
+#endif
       signal (SIGGRANT, fatal_error_signal);
       signal (SIGRETRACT, fatal_error_signal);
       signal (SIGSOUND, fatal_error_signal);
@@ -504,7 +587,7 @@ return ARG as the exit program code.")
 
   GCPRO1 (arg);
 
-  if (!NULL (Vkill_emacs_hook))
+  if (!EQ (Vkill_emacs_hook, Qnil))
     call0 (Vkill_emacs_hook);
 
   if (feof (stdin))
@@ -599,7 +682,7 @@ Take symbols from SYMFILE (presumably the file you executed to run Emacs).")
 
   CHECK_STRING (intoname, 0);
   intoname = Fexpand_file_name (intoname, Qnil);
-  if (!NULL (symname))
+  if (!EQ (symname, Qnil))
     {
       CHECK_STRING (symname, 0);
       if (XSTRING (symname)->size)
@@ -619,7 +702,7 @@ Take symbols from SYMFILE (presumably the file you executed to run Emacs).")
   malloc_init (&my_edata, malloc_warning);
 #endif
   unexec (XSTRING (intoname)->data,
-	  !NULL (symname) ? XSTRING (symname)->data : 0, &my_edata, 0, 0);
+	  !EQ (symname, Qnil) ? XSTRING (symname)->data : 0, &my_edata, 0, 0);
 #endif /* not VMS */
 
   Vpurify_flag = tem;
@@ -646,7 +729,10 @@ decode_env_path (evarname, defalt)
 
   Lisp_Object lpath;
 
-  path = (char *) egetenv (evarname);
+  if (evarname != 0)
+    path = (char *) egetenv (evarname);
+  else
+    path = 0;
   if (!path)
     path = defalt;
   lpath = Qnil;
