@@ -1,22 +1,21 @@
-/* Definitions file for GNU Emacs running on bsd 4.3
+/* Definitions file for GNU Emacs running on bsd 4.2
    Copyright (C) 1985, 1986 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
-GNU Emacs is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY.  No author or distributor
-accepts responsibility to anyone for the consequences of using it
-or for whether it serves any particular purpose or works at all,
-unless he says so in writing.  Refer to the GNU Emacs General Public
-License for full details.
+GNU Emacs is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 1, or (at your option)
+any later version.
 
-Everyone is granted permission to copy, modify and redistribute
-GNU Emacs, but only under the conditions described in the
-GNU Emacs General Public License.   A copy of this license is
-supposed to have been given to you along with GNU Emacs so you
-can know your rights and responsibilities.  It should be in a
-file named COPYING.  Among other things, the copyright notice
-and this notice must be preserved on all copies.  */
+GNU Emacs is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with GNU Emacs; see the file COPYING.  If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 
 /*
@@ -24,9 +23,9 @@ and this notice must be preserved on all copies.  */
  *	Define all the symbols that apply correctly.
  */
 
-#ifndef BSD4_3
-#define BSD4_3
-#endif /* BSD4_3 */
+#ifndef BSD4_2
+#define BSD4_2
+#endif /* BSD4_2 */
 
 #ifndef BSD
 #define BSD
@@ -43,14 +42,14 @@ and this notice must be preserved on all copies.  */
 
 /* #define NOMULTIPLEJOBS */
 
-/* Do not use interrupt_input = 1 by default, because in 4.3
-   we can make noninterrupt input work properly.  */
+/* Default is to set interrupt_input to 1: do input buffering within Emacs */
 
-#undef INTERRUPT_INPUT
+#define INTERRUPT_INPUT
 
 /* First pty name is /dev/ptyp0.  */
 
 #define FIRST_PTY_LETTER 'p'
+ 
 /*
  *	Define HAVE_TIMEVAL if the system supports the BSD style clock values.
  *	Look in <sys/time.h> for a timeval structure.
@@ -103,7 +102,7 @@ and this notice must be preserved on all copies.  */
    The alternative is that a lock file named
    /usr/spool/mail/$USER.lock.  */
 
-#define MAIL_USE_FLOCK
+/* #define MAIL_USE_FLOCK */
 
 /* Define CLASH_DETECTION if you want lock files to be written
    so that Emacs can tell instantly when you try to modify
